import itertools
import pandas as pd


class Analyzer:
    @staticmethod
    def find_single_id_candidate_columns(df: pd.DataFrame):
        total_records = df.shape[0]
        column_list = df.columns
        id_candidates = []
        for column in column_list:
            column_series = df[column]
            column_unique = column_series.unique()
            column_unique_count = column_unique.size
            if column_unique_count == total_records:
                id_candidates.append(column)
        return id_candidates

    @staticmethod
    def find_id_pair_candidates(df: pd.DataFrame):
        total_records = df.shape[0]
        column_list = df.columns
        id_candidate_pairs = []
        combo_df = pd.DataFrame()
        for column_set in itertools.combinations(column_list, 2):
            if column_set is None:
                continue
            first_column = column_set[0]
            second_column = column_set[1]
            combo_df["combo"] = df[first_column].astype(str) + df[second_column].astype(str)
            combined_unique = combo_df["combo"].unique()
            combined_unique_count = combined_unique.size
            if combined_unique_count == total_records:
                id_candidate_pairs.append(column_set)
        return id_candidate_pairs



import unittest
import pandas as pd
import numpy as np
from unittest.mock import Mock, patch
from src.etl.database.loader import Loader, ExtraColumnsException, ColumnDataException


class TestLoader(unittest.TestCase):

    @patch('src.etl.database.loader.insert_to_mssql_db')
    def test_insert_to_mssql_table(self, mock_insert):
        cursor = Mock()
        df = pd.DataFrame({
            'id': [1, 2, 3],
            'name': ['Alice', 'Bob', 'Charlie'],
            'value': [10.5, 20.3, 30.7]
        })
        schema = 'dbo'
        table = 'test_table'

        Loader.insert_to_mssql_table(cursor, df, schema, table)

        self.assertTrue(mock_insert.called)
        self.assertGreater(mock_insert.call_count, 0)

    @patch('pandas.read_sql')
    def test_validate_mssql_upload(self, mock_read_sql):
        connection = Mock()
        df = pd.DataFrame({
            'id': [1, 2, 3],
            'name': ['Alice', 'Bob', 'Charlie'],
            'value': [10.5, 20.3, 30.7]
        })
        schema = 'dbo'
        table = 'test_table'

        mock_read_sql.return_value = pd.DataFrame({
            'COLUMN_NAME': ['id', 'name', 'value'],
            'DATA_TYPE': ['int', 'varchar', 'float'],
            'CHARACTER_MAXIMUM_LENGTH': [None, 50, None],
            'NUMERIC_PRECISION': [10, None, 10]
        })

        try:
            Loader.validate_mssql_upload(connection, df, schema, table)
        except (ExtraColumnsException, ColumnDataException):
            self.fail("validate_mssql_upload raised an exception unexpectedly")


if __name__ == '__main__':
    unittest.main()
